"use strict";
/* eslint-disable import/prefer-default-export */
/* A react hook async request */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.useAsyncRequest = exports.EmptyArgs = void 0;
const react_1 = require("react");
const prop_types_1 = __importDefault(require("prop-types"));
const error_handling_1 = require("./error-handling");
const ENV = process.env.NODE_ENV;
const demoCallFn = () => __awaiter(void 0, void 0, void 0, function* () { return 'DEMO RESPONSE'; });
exports.EmptyArgs = [];
/**
 * Async React request hook
 *
 * @param {Object} {
 *   - args {Array} - the arguments to be passed to the callFn,
 *   - callFn {function} - the async function to call
 *   - cleanupFunc {function} - function to clean up the async call
 *   - callOnInit {bool}(true) - if true then will call the function on component mount
 *   - callback {function} - function to call when async call returns (response, args) => {}
 * }
 * @return {Object} {
 *   - response {any} - the response from the async call
 *   - reload {function} - function to recall the call function
 *   - call {function} - same as reload
 *   - loading {bool} - true when callfn has been called but not yet returned
 *   - hasLoaded {bool} - true after the callFn has been ran at least once
 *   - error {string} - error returned by callfn
 * }
 */
const useAsyncRequest = ({ id, args: argsInitial, // TODO: Rename to defaultArgs
callFn, cleanupFunc = () => null, callOnInit = true, debug = false, 
//callback = (response: ResponseType, args: Args) => null as any
callback: callbackIn, errorCallback: errorCallbackIn, reloadKey, }) => {
    // const [latestCallId, setLatestCallId] = useState(0);
    const [resultState, setResultState] = (0, react_1.useState)({
        isLoading: false,
        latestLoadingId: 0,
        resultsData: null,
        hasLoaded: false,
        error: undefined,
        callCount: 0,
    });
    const [args, setArgs] = (0, react_1.useState)(argsInitial || exports.EmptyArgs);
    const [callback, setCallback] = (0, react_1.useState)(() => callbackIn);
    const [allowLoad, setAllowLoad] = (0, react_1.useState)(callOnInit);
    const [forceLoad, setForceLoad] = (0, react_1.useState)(true);
    (0, react_1.useEffect)(() => {
        setCallback(() => callbackIn);
    }, [callbackIn]);
    const reload = (0, react_1.useCallback)((argsUpdated, callbackOverride) => {
        setArgs((prev) => argsUpdated || prev);
        setCallback((prev) => callbackOverride || prev);
        setAllowLoad(true);
        setForceLoad(true);
    }, []);
    (0, react_1.useEffect)(() => {
        setForceLoad(true);
    }, [reloadKey]);
    // Call callback in use effect
    (0, react_1.useEffect)(() => {
        if (forceLoad && allowLoad) {
            setForceLoad(false);
            setResultState((prev) => (Object.assign(Object.assign({}, prev), { isLoading: true, latestLoadingId: prev.latestLoadingId + 1, resultsData: null, error: undefined })));
            const newCallId = resultState.latestLoadingId + 1;
            if (!callFn)
                throw Error('Missing call fn');
            try {
                callFn
                    .apply(null, args || [])
                    .then((responseData) => {
                    setResultState((prev) => {
                        // check we are only returning the data from the most recent call
                        if (prev.latestLoadingId === newCallId) {
                            return Object.assign(Object.assign({}, prev), { isLoading: false, 
                                // TODO: reseting latest loading id could result in the first call being accepted
                                latestLoadingId: 0, resultsData: responseData, hasLoaded: true, callCount: prev.callCount + 1 });
                        }
                        return prev;
                    });
                    if (callback)
                        callback(responseData, args);
                    setArgs(() => argsInitial || exports.EmptyArgs);
                    setCallback(() => callbackIn);
                })
                    .catch((e) => {
                    if (ENV === 'development' || debug) {
                        console.log(e);
                    }
                    setResultState((prev) => {
                        if (prev.latestLoadingId === newCallId) {
                            return Object.assign(Object.assign({}, prev), { isLoading: false, 
                                // TODO: reseting latest loading id could result in the first call being accepted
                                latestLoadingId: 0, resultsData: null, hasLoaded: true, error: new error_handling_1.AsyncRequestError('Failed to load') });
                        }
                        return prev;
                    });
                    // TODO: This filtering should be performed outside this component!
                    const errorMessage = e.name === 'ApiError' ? e.message : 'Unknown Error';
                    if (errorCallbackIn)
                        errorCallbackIn(new error_handling_1.AsyncRequestError(errorMessage, e));
                });
            }
            catch (error) {
                console.error(`Async Call function failed: ${id || callFn.name}`);
                throw error;
            }
        }
        return cleanupFunc;
    }, [
        forceLoad,
        allowLoad,
        args,
        callback,
        resultState,
        callFn,
        cleanupFunc,
        debug,
        argsInitial,
        callbackIn,
        errorCallbackIn,
    ]);
    return {
        resultState,
        response: resultState.resultsData,
        reload,
        call: reload,
        loading: resultState.isLoading,
        hasLoaded: resultState.hasLoaded,
        error: resultState.error,
        callCount: resultState.callCount,
    };
};
exports.useAsyncRequest = useAsyncRequest;
exports.useAsyncRequest.propTypes = {
    args: prop_types_1.default.arrayOf(prop_types_1.default.any),
    callFn: prop_types_1.default.func.isRequired,
    cleanupFunc: prop_types_1.default.func,
    callOnInit: prop_types_1.default.bool,
    debug: prop_types_1.default.bool,
    callback: prop_types_1.default.func,
};
//# sourceMappingURL=use-async-request.js.map